unit CompInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ShellAPI, FileCtrl;

type
  TFormCompareInfo = class(TForm)
    btnOK: TButton;
    Panel1: TPanel;
    Image1: TImage;
    lblName: TLabel;
    Bevel3: TBevel;
    Label1: TLabel;
    lblType: TLabel;
    GroupBoxNew: TGroupBox;
    Label2: TLabel;
    lblNewDir: TLabel;
    Label3: TLabel;
    lblNewSize: TLabel;
    Label7: TLabel;
    lblNewTime: TLabel;
    GroupBox1: TGroupBox;
    Label4: TLabel;
    lblOldDir: TLabel;
    Label6: TLabel;
    lblOldSize: TLabel;
    Label9: TLabel;
    lblOldTime: TLabel;
    Label5: TLabel;
    lblSizeDiff: TLabel;
    procedure btnOKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    { Private declarations }
    hIcon: THandle;
  public
    { Public declarations }
    FileName: string;
    OldDir: string;
    NewDir: string;
  end;

implementation

{$R *.DFM}

procedure TFormCompareInfo.btnOKClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;


procedure TFormCompareInfo.FormShow(Sender: TObject);

const
  na = 'not available';

var
  search: TSearchRec;
  s: string;
  ext: string;
  NameOfFileThatExists,
  OldFileName: string;
  ShInfo: TShFileInfo;
  SizeDiff, OldSize, NewSize: LongInt;

begin
  if NewDir <> '' then
  begin
    if NewDir[length(NewDir)] <> '\' then NewDir := NewDir + '\';
    NewDir := NewDir + ExtractFilePath (FileName);
    if NewDir[length(NewDir)] <> '\' then NewDir := NewDir + '\';
  end;
  if OldDir <> '' then
  begin
    if OldDir[length(OldDir)] <> '\' then OldDir := OldDir + '\';
    OldDir := OldDir + ExtractFilePath (FileName);
    if OldDir[length(OldDir)] <> '\' then OldDir := OldDir + '\';
  end;
  FileName := ExtractFileName(FileName);
  OldFileName := OldDir + FileName;
  FileName := NewDir + FileName;

  if OldDir = '' then OldDir := na;
  if NewDir = '' then NewDir := na;
  lblOldDir.caption := MinimizeName(OldDir, Canvas, GroupBox1.Width - lblOldDir.Left);
  lblNewDir.Caption := MinimizeName(NewDir, Canvas, GroupBox1.Width - lblNewDir.Left);

  NameOfFileThatExists := '';
  OldSize := 0;
  NewSize := 0;

  // we *must* have a real file to open.....
  if FindFirst(OldFileName, faAnyFile, search) = 0
    then
  begin
    with search, search.FindData do
    begin
      NameOfFileThatExists := OldFileName;
      lblOldDir.Hint := lblOldDir.Caption;
      lblOldDir.ShowHint := True;
      OldSize := Size;
      if size < 1024
        then s := IntToStr(Size) + ' Bytes'
      else
        if size < 1048576
          then s := Format('%.2n KB', [Size / 1024])
        else s := Format('%.2n MB', [Size / 1048576]);
      lblOldSize.Caption := s + Format(' (%.0n Bytes)', [Size + 0.0]);
      lblOldTime.Caption := FormatDateTime('c', FileDateToDateTime(Time));
    end;
  end else
  begin
    lblOldSize.Caption := na;
    lblOldTime.Caption := na;
  end;
  FindClose(search); // oldfilename

  // we *must* have a real file to open.....
  if FindFirst(FileName, faAnyFile, search) = 0
    then
  begin
    with search, search.FindData do
    begin
      NameOfFileThatExists := FileName;
      lblNewDir.Hint := lblNewDir.Caption;
      lblNewDir.ShowHint := True;
      NewSize := Size;
      if size < 1024
        then s := IntToStr(Size) + ' Bytes'
      else
        if size < 1048576
          then s := Format('%.2n KB', [Size / 1024])
        else s := Format('%.2n MB', [Size / 1048576]);
      lblNewSize.Caption := s + Format(' (%.0n Bytes)', [Size + 0.0]);
      lblNewTime.Caption := FormatDateTime('c', FileDateToDateTime(Time));
    end;
  end else
  begin
    lblNewSize.Caption := na;
    lblNewTime.Caption := na;
  end;
  FindClose(search);

  hIcon := 0;

  lblName.Caption := ExtractFileName(NameOfFileThatExists);
  if lblName.Caption = '' then lblName.Caption := 'file ' + na;
  Caption := Caption + lblName.Caption;
  
  // If either the old file or the new file exists, use it to get the icon and file type
  if NameOfFileThatExists <> '' then
  begin
    s := '';
    if ShGetFileInfo(PChar(NameOfFileThatExists), 0, ShInfo, SizeOf(ShInfo),
      SHGFI_ICON or SHGFI_LARGEICON or SHGFI_TYPENAME) <> 0 then
    begin
      Image1.Picture.Icon.Handle := ShInfo.hIcon;
      s := ShInfo.szTypeName;
    end;

    ext := ExtractFileExt(NameOfFileThatExists); // get the file extension
    if s = '' then // not known by the registry so just
    begin // call it an EXT file (or whatever)
      Delete(ext, 1, 1);
      s := UpperCase(ext) + '-file';
    end;
    lblType.Caption := s;
  end
  else
    lblType.Caption := na;

  Sizediff := NewSize - OldSize;
  if Abs(Sizediff) < 1024
    then s := IntToStr(Sizediff) + ' Bytes'
  else
    if Sizediff < 1048576
      then s := Format('%.2n KB', [Sizediff / 1024])
    else s := Format('%.2n MB', [Sizediff / 1048576]);
  lblSizeDiff.Caption := s + Format(' (%.0n Bytes)', [Sizediff + 0.0]);

end;


procedure TFormCompareInfo.FormHide(Sender: TObject);
begin
  if hIcon <> 0 then DestroyIcon(hIcon);
end;

end.

