{  Main form for Update

  Copyright (c) 1998 Gregory L. Bullock (bullock@tsppd.com).
  Freeware: May be freely distributed and modified. Use at your own risk.
  This program draws heavily on

    - the aDiff unit, Copyright (c) 1997 S.Kurinny & S.Kostinsky

    - the Lh5Unit unit, which had various contributers, as noted in its
      source code

  History:
    23-7-01 Thanks to Sascha Adams (Sascha@Adams1.DE) for improvements to the
            auto-updating capabilities.  Update now accepts a 5th (and 6th)
            parameter on the command line, the 5th parameter is either a window
            handle or a window class name, and the optional 6th parameter is
            'handle' if the 5th parameter is to be interpreted as a window
            handle. Update will wait for up to 30 seconds until the given window
            handle is closed or all windows with the given window class name are
            closed, then it will apply the update file and restart the application.

            Also modified to allow skipping the registry key containing the
            last-applied update file (by setting the third command line parameter
            to '@').

     4-5-01 Added ability to store multiple version updates in a single .Upd
            file.  That is, a single file can contain
              ver. 1 -> ver. 2 updates,
              ver. 2 -> ver. 3 updates,
              . . .
              ver. n-1 -> ver. n updates
            This file can be applied to "old" files of any version and it will
            change them to ver. n.

            Thanks to Denver Jeans (denver@dynamic.net.au) for various changes:
            Changed the way progress reporting is done to cope with very large files.
            Made the Completeds and Totals into properties.
            Added Quiet_Mode processing.
            If cannot do the rename after changing an old file into a new, give
            option of trying to rename to New_<fname> just in case it is an
            in-use EXE file - then someone else can rename again later without
            any problems.

            Thanks to Roland Ruder (r.ruder@gmx.de) for various cosmetic
            and performance improvements.

     2-5-01 Made the LogForm sizeable.
     
    14-4-01 Changed TMemoryStream variables to TTemporaryFileStream (with
            temporary file names) so we're less likely to run out of memory.
            Shortened the current filename displayed in the progress window,
            as needed, to fit the available space.

            Thanks to Denver Jeans (denver@dynamic.net.au) for developing the
            TTemporaryFileStream class (adapted by Greg Bullock) and for
            reporting and helping diagnose a bug in handling large .Upd files.

    16-2-01 Thanks to Boris Nienke <info@nsonic.de> for fixing a bug when
            attempting to add a new file in a directory that doesn't exist
            on the target machine.

   20-11-00 Thanks to Michael Tien <mtien@magnetargames.com> for fixing
            a problem when attempting to rename a file in a directory that
            doesn't happen to exist on the target machine.

   14-12-99 Added ability to copy over any existing "old" file without
            first verifying that it passes the CRC version checking. This
            is equivalent to deleting the old file then adding the new one.

   18-11-99 Thanks to Jouko Tolonen for the following changes (adapted by
            Greg Bullock):
            Initialized OpenDialog.InitialDir more intelligently in
            BrowseChangesFileClick.
            Disabled GoBtn by default, enabling only when ChangesFile has
            the .Upd extension.

     4-5-99 Added option to update-all-or-nothing in case of failure.

    21-4-99 Thanks to Kurt Senfer for improving the organization of
            ApplyUpdateFile and for making changes to enable an auto-updating
            system, where some Auto-Updating applications on start-up test if
            there are pending updates (one or more), and prompt the user to
            say Yes or No to run an automatic update.  See the Help file for
            more information on using auto-update.

     3-4-99 Retired support for 16-bit compilation; it was getting too
            troublesome.
            Set the date/time of added and updated files to match the original.
            Add ability to add new files or delete obsolete files.
            Use the file sizes in addition to the CRCs to help identify files.
            Show progress bar for all files in addition to progress bar for
            the current file.

            Thanks to Jens Liebermann for figuring out how to maintain D2
            compatibility.

   28-10-98 Made Progress dialog get updated more frequently under 32-bits.

     3-7-98 Added a CRC for the *compressed* differences to better detect
            whether an .Upd is corrupted.

    16-5-98 Modified to compile under D3 (32-bits) as well as D1 (16-bits).

            Added a new compiler directive, MAKE16AND32COMPATIBLE, which
            will ensure that the *.Upd file is compatible independent
            of the platform (16- or 32-bit) which creates it or uses it.
            On the one test case I ran, I found that using MAKE16AND32COMPATIBLE
            increased the size of the *.Upd file.  The file sizes I got were
              29304  for the 32-bit .Upd file
              30746  for the 16-bit .Upd file
              30759  for the compatible .Upd file

            Improved one of the log messages and corrected the help file.

            Thanks to Jeremy Coulter for making changes so Update runs
            automatically when the user double clicks on a .Upd file in the
            Windows Explorer (provided Update is associated with the .Upd
            extension).

            Thanks to Massimiliano Orso, Kurt Senfer, Jeremy Coulter, Mavarik,
            Holger Dors, and Fabrice Avaux for reporting bugs and bug fixes.

    23-4-98 Initial version.

  If you fix any bugs or make significant enhancements, I ask you to send
  me your modifications.

  For example, at present, these programs support an auto-updating system over
  a LAN or WAN, but not over the WEB.  Perhaps a future version of these
  programs will enable Update to support auto-updating system over the WEB.

  If you make any changes to the structure of the Update File,
  change the UpdateFileHeader in the Common Unit so older versions
  of Update won't try to read an Update File that they won't understand.
}

unit UpdateFm;

interface

uses
  SysUtils, Windows, About, DoUpdate, UpdateGlobals, Forms, ExtCtrls,
  FileCtrl, StdCtrls, Dialogs, Classes, Controls, Buttons;

type
  TUpdateForm = class(TForm)
    GoBtn: TBitBtn;
    CancelBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    GroupBox1: TGroupBox;
    TargetFile: TLabel;
    TargetDirectoryListBox: TDirectoryListBox;
    DriveComboBox1: TDriveComboBox;
    GroupBox2: TGroupBox;
    ChangesFile: TEdit;
    BrowseChangesFile: TButton;
    AboutBtn: TBitBtn;
    Timer1: TTimer;
    procedure BrowseChangesFileClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    procedure GoBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ChangesFileChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


var
  UpdateForm: TUpdateForm;

implementation

{$R *.DFM}

Uses KSAutoUpdate,WaitMsg; //needed for the enumwindowscall from the Timer

procedure TUpdateForm.BrowseChangesFileClick(Sender: TObject);
begin
  if ChangesFile.Text = '' then
    OpenDialog.InitialDir := ExtractFileDir(Application.ExeName)
  else
  begin
    OpenDialog.FileName := ExtractFileName(ChangesFile.Text);
    OpenDialog.InitialDir := ExtractFileDir(ChangesFile.Text);
  end;

  if OpenDialog.Execute then
    ChangesFile.Text := AnsiLowerCase(OpenDialog.FileName);
end;

procedure TUpdateForm.AboutBtnClick(Sender: TObject);
begin
  AboutBox := TAboutBox.Create(Self);
  AboutBox.ShowModal;
  AboutBox.Free;
  AboutBox := nil;
end;

procedure TUpdateForm.GoBtnClick(Sender: TObject);
begin
	Targetdir := AnsiLowerCase(TargetDirectoryListBox.Directory);

	if Targetdir = ''
		then begin
  		TargetDir := GetCurrentDir;
  		TargetDirectoryListBox.Directory := Targetdir;
  	end;
  if Targetdir[Length(Targetdir)] <> '\'
    then Targetdir := Targetdir + '\';

  ApplyUpdateFile(ChangesFile.Text);
end;

procedure TUpdateForm.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TUpdateForm.FormShow(Sender: TObject);
begin
  Show;
  //KS obsolete under win32 -> Application.ProcessMessages;
  if ParamStr(1) > '' then
  begin
   ApplyUpdateFile(ParamStr(1));
   Application.Terminate;
  end;
end;

procedure TUpdateForm.ChangesFileChange(Sender: TObject);
begin
  GoBtn.Enabled := AnsiCompareText(ExtractFileExt(ChangesFile.Text), '.'+OpenDialog.DefaultExt) = 0;
end;

procedure TUpdateForm.Timer1Timer(Sender: TObject);
begin
 If Foundwindowclass
  Then Begin
        foundwindowclass := False;
        If waittimedout > 0
         Then Begin
               Dec(waittimedout);
               EnumWindows(@KSAutoUpdate.EnumWinProc, self.Handle);
              End;
        WaitMessage.label2.caption := InttoStr(waittimedout);
       End
  Else Begin
        Timer1.enabled := False;
        waitwindowclose := False;
       End;
end;


end.
